#include "TCanvas.h"
#include "TPolyMarker3D.h"
#include "TPointSet3D.h"
#include "TRandom3.h"
#include "TGeoHelix.h"
#include "TStyle.h"
#include "TView.h"
#include "TStopwatch.h"
   
TCanvas*       g_canvas;
TPolyMarker3D* g_pm;
TPointSet3D*   g_ps;
TRandom3 g_rnd;

void gltracks(const Int_t N=500)
{
  // Fake N-tracks with points.
  //
  // Use flat curvature/z-step distribution.
  // Step by 1cm until R_max, Z_max is reached or number of points
  // exceeds N_max_points

  const Int_t    N_max_points = 1000;
  const Double_t R_max        = 2;
  const Double_t Z_max        = 3;

  const Double_t R_max_sqr    = R_max*R_max;

  Int_t N_all = 0;


  g_pm = new TPolyMarker3D(100*N);
  g_ps = new TPointSet3D(200*N);

  Double_t curv=1, zstep=5, phi, dirx, diry, dirz;
  Int_t charge;
  const Double_t* p = 0;

  TGeoHelix h(1,1);
  
  int j;
  for(int i=0; i<N; ++i) {

    curv  = g_rnd.Uniform(0,2);
    zstep = g_rnd.Uniform(0,10);
    phi   = g_rnd.Uniform(0,TMath::TwoPi());
    charge= 1;
    if (g_rnd.Rndm() > 0.5) charge = -1;
    
    h.SetXYcurvature(curv);
    h.SetHelixStep(zstep);
    h.SetCharge(charge);
    h.SetField(0, 0, 1);
    h.InitPoint(g_rnd.Uniform(-0.1,0.1),
		g_rnd.Uniform(-0.1,0.1),
		g_rnd.Uniform(-0.1,0.1));
    dirx = TMath::Cos(phi)/curv;
    diry = TMath::Sin(phi)/curv;
    dirz = zstep/curv;
    if (g_rnd.Rndm() > 0.5) dirz = -dirz;
    h.InitDirection(dirx,diry,dirz,kFALSE);
if(N_all==0) printf("dirx=%g, diry=%g, dirz=%g\n",dirx,diry,dirz);
    p = h.GetCurrentPoint();
    h.Step(0.0001); // required to get a valid p[]
    if (i==0) h.Dump();
    
    for(j=0; j<N_max_points; ++j) {
if (N_all < 4) printf("i=%d, j=%d, p= %g, %g, %g\n",i,j,p[0],p[1],p[2]);
      g_pm->SetPoint(N_all,p[0], p[1], p[2]);
      g_ps->SetPoint(N_all,p[0], p[1], p[2]);
      N_all++;

      h.Step(0.01); //step 1 cm along the helix
      if(TMath::Abs(p[2]) > Z_max || p[0]*p[0]+p[1]*p[1] > R_max_sqr)
	break;
    }
  }
  
  printf("Created %d tracks, number of all points %d.\n", N, N_all);

  g_canvas = new TCanvas("glcanvas");;
  g_canvas->Divide(2, 1);
  
  double range = 3;
  g_canvas->cd(1);
  char *viewer = "ogl";  //you can try "gl" in the pad  or "" for pad graphics
  if (gROOT->IsBatch()) viewer = "";
  TView *view1 = new TView(1);
  view1->SetRange( 0, 0, -2*range, range, range, 2*range );
  gPad->GetViewer3D(viewer); // Force creation of OpenGL viewer
  g_pm->Draw(viewer);
  
  g_canvas->cd(2); 
  TView *view2 = new TView(1);
  view2->SetRange( 0, 0, -2*range, range, range, 2*range );
  gPad->GetViewer3D(viewer);
  g_ps->Draw(viewer);

  printf("Call update() to test refresh speeds.\n");
}

void update()
{
  TStopwatch t;

  t.Start();
  g_pm->SetMarkerColor(Color_t(1 + 6*g_rnd.Rndm()));
  g_canvas->cd(1);
  gPad->Modified();
  gPad->Update();
  t.Stop();
  printf("PolyMarker %lf\n", t.RealTime());

  t.Start();
  g_ps->SetMarkerColor(Color_t(1 + 6*g_rnd.Rndm()));
  g_canvas->cd(2);
  gPad->Modified();
  gPad->Update();
  t.Stop();
  printf("PointSet   %lf\n", t.RealTime());
}
